-- filename : type-lam.lua
-- comment  : Pretty printing of (extended) lambda calculus
-- author   : Matthijs Kooijman, Universiteit Twente, NL
-- copyright: Matthijs Kooijman
-- license  : None

local utf = unicode.utf8

if not buffers                 then buffers                 = { } end
if not buffers.visualizers     then buffers.visualizers     = { } end
if not buffers.visualizers.lam then buffers.visualizers.lam = { } end

buffers.visualizers.lam.colors = {
    "prettytwo",
    "prettyone",
    "prettythree",
    "prettyfour"
}

buffers.visualizers.lam.symbols = {
    [' '] = {repr = '\\obs '},
    ['_'] = {repr = '\\_'},
    ['->'] = {repr = '\\rightarrow'},
    -- The default * sits very high above the baseline, \ast (u+2217) looks
    -- better.
    ['*'] = {repr = '\\ast'},
}

buffers.visualizers.lam.keywords = {
    ['case'] = {},
    ['of'] = {},
    ['let'] = {},
    ['in'] = {},
}


function buffers.visualizers.lam.take_symbol(str)
    -- See if str starts with a symbol, and return the remaining
    -- string and that symbol. If no symbol from the table is matched,
    -- just returns the first character.
    -- We can do a lookup directly, since symbols can be different in
    -- length, so we just loop over all symbols, trying them in turn.
    for symbol,props in pairs(buffers.visualizers.lam.symbols) do
        -- Try to remove symbol from the start of str 
        symbol, newstr = utf.match(str, "^(" .. symbol .. ")(.*)")
        if symbol then
            -- Return this tokens repr, or just the token if it has no
            -- repr.
            res = props.repr or symbol
            -- Enclose the token in {\style .. }
            if props.style then
                res = "{\\" .. props.style ..  " " .. res ..  "}"
            end
            return res, newstr
        end
    end
    -- No symbol found, just return the first character
    return utf.match(str, "^(.)(.*)")
end

-- Take a single word from str, if posible. Returns the rest of the string and
-- the word taken.
function buffers.visualizers.lam.take_word(str)
        res, newstr = utf.match(str, "^(%a+)(.*)")
        return res, newstr or str
end

function buffers.visualizers.lam.flush_line(str,nested)
    local result, state = { }, 0
    local finish, change = buffers.finish_state, buffers.change_state
    local take_symbol = buffers.visualizers.lam.take_symbol
    local take_word = buffers.visualizers.lam.take_word
    -- Set the colorscheme, which is used by finish_state and change_state
    buffers.currentcolors = buffers.visualizers.lam.colors

    while str ~= "" do
        local found = false
        local res
        -- See if the next token is a word
        res, str = take_word(str)
        if res then
            if buffers.visualizers.lam.keywords[res] then
                -- Make all keywors bold
                res = "{\\bold " .. res ..  "}"
            end
            -- All other words are unmodified
        else
            -- The next token is not a word, it must be a symbol
            res, str = take_symbol(str)
        end

        -- Append the resulting token
        result[#result+1] = res
    end

    state = finish(state, result)
    buffers.flush_result(result,nested)
end

-- vim: set sw=4 sts=4 expandtab ai:
